/*
	$Id: component_options.h,v 1.29 2001/09/22 15:52:15 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanGUI="Framework"
//! header=gui.h

#ifndef header_styleoptions
#define header_styleoptions

#include "../Core/System/clanstring.h"
#include "../Core/System/error.h"
#include <map>
#include <string>
#include <stdio.h>

//: Sets the options for components
class CL_ComponentOptions
{
public:
//! Attributes:
	//: Exists
	bool exists(const std::string &option_name) const
	{
		return options.find(option_name) != options.end();
	}

	//: Count
	int count(const std::string &option_name) const
	{
		return options.count(option_name);
	}

	//: Getting the value of a unexisting option is undefined, so use exists 
	//: first to check if the option exists
	const std::string &get_value(const std::string &option_name, int offset=0) const
	{ 
		std::multimap<std::string, std::string>::const_iterator it = options.find(option_name);
		if (it == options.end())
		{
			char buf[100];
			sprintf(buf, "Missing component option '%s'", option_name.c_str());
			throw CL_Error(buf);
		}
		for (;offset>0;offset--)
		{
			it++;
			if (it == options.end())
			{
				char buf[100];
				sprintf(buf, "Missing component option '%s'", option_name.c_str());
				throw CL_Error(buf);
			}
		}
		return (*it).second; 
	}

	//: Get value as int
	int get_value_as_int(const std::string &option_name, int offset=0) const
	{
		const std::string &val = get_value(option_name, offset);
		return atoi(val.c_str()); 
	}

	//: Get value as bool
	bool get_value_as_bool(const std::string &option_name, int offset=0) const
	{
		const std::string &val = get_value(option_name, offset);
		CL_String s(val);
		s.to_lower();

		if(s == "yes" ||
			s == "true" ||
			s == "on" ||
			s == "1")
			return true;
		else if(s == "no" ||
			s == "false" ||
			s == "off" ||
			s == "0")
			return false;

		char buf[200];
		sprintf(
			buf,
			"Invalid bool value '%s' in component option '%s'", 
			val.c_str(),
			option_name.c_str());
		throw CL_Error(buf);
	}
	
//! Operations:
	//: Add option
	void add_option(const std::string &option_name, const std::string &option_value)
	{
		options.insert(std::pair<std::string, std::string>(option_name, option_value));
	}
	
	//: Add option
	void add_option(const std::string &option_name, int option_value)
	{
		CL_String value; value << option_value;
		add_option(option_name, value);
	}
	
	//: Clear
	void clear()
	{
		options.clear();
	}

	//: Options
	std::multimap<std::string, std::string> options;
};

#endif
