/*
	$Id: netobject.h,v 1.15 2001/10/10 11:50:27 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanNetwork="NetObjects"
//! header=network.h

#ifndef header_netobject
#define header_netobject

#include <string>
#include "../signals.h"

class CL_NetObject_Generic;

//: Network replication object.
//- The netobject is a message dispatcher for objects being replicated over the network.
//- <p>It provides an easy system for a server object to send messages to replicated clients
//- on remote computers, as well as getting answers back.</p>
class CL_NetObject
{
//! Construction:
public:
	//: Copy constructor.
	CL_NetObject(const CL_NetObject &copy);

	//: <p>Construct a new netobject, assigning it an unique handle that identifies it over the
	//: network. Use this constructor to create a server side netobject.</p>
	CL_NetObject(class CL_NetObjectChannel *obj_channel);

	//: <p>Construct a netobject, assigning it the object handle obj_id. This is normally used
	//: to construct a client side netobject.</p>
	CL_NetObject(int obj_id, class CL_NetObjectChannel *obj_channel);

	//: NetObject destructor
	virtual ~CL_NetObject();

//! Attributes:
public:
	//: Returns the netobject handle that identifies it over the network.
	int get_obj_id() const;

//! Operations:
public:
	//: Connect the slot to the specified message sub-type.
	CL_Slot connect(int msg_type, CL_Slot_v1<class CL_InputSource &> *slot);

	//: Connect the slot to the specified talkback message sub-type.
	CL_Slot connect(int talkback_type, CL_Slot_v2<const class CL_NetComputer &, class CL_InputSource &> *slot);

	//: Send a message to the client objects.
	void send(int msg_type, const std::string &message);

	//: Send a message to the server objects.
	void talkback(int talkback_type, const std::string &message);

//! Implementation:
private:
	CL_NetObject_Generic *impl;
};

#endif
