/*
	$Id: signal_v3.h,v 1.9 2001/09/22 15:52:17 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanSignal="System"
//! header=signals.h

#ifndef header_signal_v3
#define header_signal_v3

#ifdef WIN32
#pragma warning ( disable : 4786 )
#endif

#include "slot_v3.h"
#include <list>

template <class PARAM1, class PARAM2, class PARAM3>
//: Signal V3 Class
class CL_Signal_v3
{
public:
	typedef CL_Slot_v3<PARAM1, PARAM2, PARAM3> *Slot;
	typedef std::list<Slot>::iterator SlotIterator;

//! Construction:
public:
	//: Signal V3 Constructor
	CL_Signal_v3()
	{
	}

	//: Signal V3 Destructor
	~CL_Signal_v3()
	{
		for (SlotIterator slot_it = slots.begin(); slot_it != slots.end(); slot_it++)
		{
			Slot slot = *slot_it;
			slot->release_signal_ref();
		}
	}

//! Operations:
public:
	//: Operator
	void operator() (PARAM1 param1, PARAM2 param2, PARAM3 param3)
	{
		call(param1, param2, param3);
	}

	//: Call
	void call(PARAM1 param1, PARAM2 param2, PARAM3 param3)
	{
		std::list<SlotIterator> remove_slots;

		// call slots connected to signal:
		for (SlotIterator slot_it = slots.begin(); slot_it != slots.end(); slot_it++)
		{
			Slot slot = *slot_it;

			// skip slot if it has no more references left in application.
			// (make it pending for removal)
			if (slot->get_slot_ref() == 0)
			{
				remove_slots.push_back(slot_it);
				continue;
			}
			
			slot->call(param1, param2, param3);
		}

		// remove all slots no longer connected to any CL_Slot.
		std::list<SlotIterator>::iterator remove_it;
		for (remove_it = remove_slots.begin(); remove_it != remove_slots.end(); remove_it++)
		{
			Slot slot = **remove_it;
			slot->release_signal_ref();
			slots.erase(*remove_it);
		}
	}

	//: Connect
	CL_Slot connect(Slot slot)
	{
		slot->add_signal_ref();
		slots.push_back(slot);
		return CL_Slot(slot);
	}

	//: Disconnect
	void disconnect(CL_Slot &disconnect_slot)
	{
		for (SlotIterator slot_it = slots.begin(); slot_it != slots.end();)
		{
			Slot slot = *slot_it;
			if (disconnect_slot.impl == slot)
			{
				slot->release_signal_ref();
				slot_it = slots.erase(slot_it);
			}
			else
				slot_it++;
		}
	}

//! Implementation:
private:
	std::list<Slot> slots;
};

#endif
