/*
	$Id: slot_v0.h,v 1.10 2001/09/22 15:52:17 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanSignal="System"
//! header=signals.h

#ifndef header_slot_v0
#define header_slot_v0

#include "slot_generic.h"
#include <list>

//: Slot V0 Class
class CL_Slot_v0 : public CL_Slot_Generic
{
public:
	//: Call
	virtual void call(class CL_SlotParent_v0 &super)=0;
};

//: Slot Parent V0 Class
class CL_SlotParent_v0
{
public:
	typedef CL_Slot_v0 *Slot;
	typedef std::list<Slot>::iterator SlotIterator;

//! Construction:
public:
	
	//: Slot Parent V0 Constructor
	CL_SlotParent_v0(const SlotIterator &pos, const SlotIterator &end)
	: pos(pos), end(end)
	{
	}

//! Operations:
public:
	//: Operator
	void operator() ()
	{
		if (pos == end) return;

		SlotIterator next = pos;
		next++;

		CL_SlotParent_v0 super(next, end);
		(*pos)->call(super);
	}

//! Implementation:
private:
	SlotIterator pos, end;
};

//: FunctionSlot V0 Class
class CL_FunctionSlot_v0 : public CL_Slot_v0
{
public:
	typedef void (*Callback)();
	typedef void (*VirtualCallback)(CL_SlotParent_v0 &);

	//: FunctionSlot V0 Constructor
	CL_FunctionSlot_v0(Callback callback, VirtualCallback virtual_callback)
	: callback(callback), virtual_callback(virtual_callback) { return; }

	//: Call
	virtual void call(CL_SlotParent_v0 &super)
	{
		if (callback) { super(); callback(); }
		else virtual_callback(super);
	}

private:
	Callback callback;
	VirtualCallback virtual_callback;
};

//: Create Slot
inline static CL_Slot_v0 *CL_CreateSlot(void (*callback)())
{
	return new CL_FunctionSlot_v0(callback, 0);
}

//: Create Virtual Slot
inline static CL_Slot_v0 *CL_CreateVirtualSlot(void (*virtual_callback)(CL_SlotParent_v0 &))
{
	return new CL_FunctionSlot_v0(0, virtual_callback);
}

template <class CallbackClass>
//: Method Slot V0 Class
class CL_MethodSlot_v0 : public CL_Slot_v0
{
public:
	typedef void (CallbackClass::*Callback)();
	typedef void (CallbackClass::*VirtualCallback)(CL_SlotParent_v0 &);

	//: Method Slot V0 Constructor
	CL_MethodSlot_v0(CallbackClass *cb_class, Callback callback, VirtualCallback virtual_callback)
	: cb_class(cb_class), callback(callback), virtual_callback(virtual_callback) { return; }

	//: Call
	virtual void call(CL_SlotParent_v0 &super)
	{
		if (callback) { super(); (cb_class->*callback)(); }
		else (cb_class->*virtual_callback)(super);
	}

private:
	CallbackClass *cb_class;
	Callback callback;
	VirtualCallback virtual_callback;
};

template <class CallbackClass>
//: Create Slot
CL_Slot_v0 *CL_CreateSlot(CallbackClass *cb_class, void (CallbackClass::*callback)())
{
	return new CL_MethodSlot_v0<CallbackClass>(cb_class, callback, 0);
}

template <class CallbackClass>
//: Create Virtual Slot
CL_Slot_v0 *CL_CreateVirtualSlot(CallbackClass *cb_class, void (CallbackClass::*virtual_callback)(CL_SlotParent_v0 &))
{
	return new CL_MethodSlot_v0<CallbackClass>(cb_class, 0, virtual_callback);
}

template <class CallbackClass, class UserData>
//: User Data Methane Slot V0 Class
class CL_UserDataMethodSlot_v0 : public CL_Slot_v0
{
public:
	typedef void (CallbackClass::*Callback)(UserData);
	typedef void (CallbackClass::*VirtualCallback)(CL_SlotParent_v0 &, UserData);

	//: User Data Methane Slot V0 Constructor
	CL_UserDataMethodSlot_v0(CallbackClass *cb_class, Callback callback, VirtualCallback virtual_callback, UserData user_data)
	: cb_class(cb_class), callback(callback), virtual_callback(virtual_callback), user_data(user_data) { return; }

	//: Call
	virtual void call(CL_SlotParent_v0 &super)
	{
		if (callback) { super(); (cb_class->*callback)(user_data); }
		else (cb_class->*virtual_callback)(super, user_data);
	}

private:
	CallbackClass *cb_class;
	Callback callback;
	VirtualCallback virtual_callback;
	UserData user_data;
};

template <class CallbackClass, class UserData>
//: Create Slot
CL_Slot_v0 *CL_CreateSlot(
	CallbackClass *cb_class,
	void (CallbackClass::*callback)(UserData),
	UserData user_data)
{
	return new CL_UserDataMethodSlot_v0<CallbackClass, UserData>(cb_class, callback, 0, user_data);
}

template <class CallbackClass, class UserData>
//: Create Virtual Slot
CL_Slot_v0 *CL_CreateVirtualSlot(
	CallbackClass *cb_class,
	void (CallbackClass::*virtual_callback)(CL_SlotParent_v0 &, UserData),
	UserData user_data)
{
	return new CL_UserDataMethodSlot_v0<CallbackClass, UserData>(cb_class, 0, virtual_callback, user_data);
}

#endif
